import router from "../router";
import { identityPaths, identityQueryParameters, userManager } from "../services/authentication";
import { addBearerTokenInterceptor, apiHttp, removeBearerTokenInterceptor } from "../services/http";
import { get, put } from "../services/persistence";

const identity = {
    state: () => ({
        user: null,
        loadingLayers: 0,
        callbackPath: null,
        registrationEnabled: false,
    }),
    getters: {
        isAuthenticated(state) {
            return state.user ? !state.user.expired : false;
        },
        isIdentityLoading(state) {
            return state.loadingLayers > 0;
        },
        username(state) {
            return (state.user && state.user.profile.name) ? state.user.profile.name : null;
        },
        authCallbackLocation(state) {
            return state.callbackPath ? state.callbackPath : get("callbackPath");
        },
    },
    mutations: {
        login(state, { user }) {
            if (!user) return;
            state.user = user;
            addBearerTokenInterceptor(user.access_token);
        },
        logout(state) {
            state.user = null;
            removeBearerTokenInterceptor();
        },
        beginAuthenticating(state) {
            state.loading += 1;
        },
        endAuthenticating(state) {
            state.loading -= 1;
        },
        authCallbackLocation(state) {
            state.callbackPath = window.location.pathname;
            put("callbackPath", state.callbackPath);
        },
        enableIdentificationRegistration(state) {
            state.registrationEnabled = true;
        },
        disableIdentificationRegistration(state) {
            state.registrationEnabled = false;
        }
    },
    actions: {
        async loadUser(context) {
            context.commit("beginAuthenticating");
            const user = await userManager.getUser();
            if (user != null) {
                context.commit("login", { user });
            }
            context.commit("endAuthenticating");
        },
        async attemptSilentAuthentication(context, options) {
            if (context.getters.isAuthenticated) return;
            context.commit("beginAuthenticating");
            context.dispatch("loadUser");
            if (!context.getters.isAuthenticated) {
                try {
                    const user = await userManager.signinSilent({
                        redirect_uri: window.location.origin + "/authentication/silent-login-callback.html"
                    });

                    context.commit("login", { user });
                } catch { }
                context.commit("endAuthenticating");
            }

            if (options && options.redirect) {
                router.replace(context.getters.authCallbackLocation);
            }
        },
        async beginAuthentication(context) {
            if (context.getters.isAuthenticated) return;
            context.commit("beginAuthenticating");
            context.dispatch("loadUser");
            if (!context.getters.isAuthenticated) {
                context.commit("authCallbackLocation");
                userManager.signinRedirect();
            }
        },
        async beginDeauthentication(context) {
            if (!context.getters.isAuthenticated) return;
            context.commit("beginAuthenticating");
            context.commit("authCallbackLocation");
            userManager.signoutRedirect();
        },
        async completeAuthentication(context, { user }) {
            if (!user) return;
            context.commit("login", { user });
            context.commit("endAuthenticating");
            router.replace(context.getters.authCallbackLocation);
        },
        async completeDeauthentication(context) {
            context.commit("logout");
            try {
                await userManager.removeUser();
            } catch (error) {
                console.error(error);
            }
            context.commit("endAuthenticating");
            router.replace(context.getters.authCallbackLocation);
        },
        beginRegistration(context) {
            context.commit("authCallbackLocation");
            window.location.replace(window.location.origin + identityPaths.Register + "?" + identityQueryParameters.ReturnUrl + "=" + "/authentication/silent-login");
        },
        async updatePublicApiSettings(context) {
            try {
                const settings = (await apiHttp.get("PublicApiSettings")).data;
                if (settings.RegistrationEnabled === "True") {
                    context.commit("enableIdentificationRegistration");
                } else {
                    context.commit("disableIdentificationRegistration");
                }
            } catch (error) {
                console.error(error);
            }
        }
    },
};

export { identity };
